using System;
using WebSites;

/// <summary>
///    User Interface for managing Web Sites.
/// </summary>
/// <remarks>This class has the following members:
///    <para> Methods:
///       <list>AddSite<see cref="AddSite"/></list>
///       <list>DeleteSite<see cref="DeleteSite"/></list>
///       <list>ModifySite<see cref="ModifySite"/></list>
///       <list>ViewSites<see cref="ViewSites"/></list>
///       <list>DisplayShortList
///            <see cref="DisplayShortList"/></list>
///    </para>
/// </remarks>
class SiteManager
{
    /// <summary>
    /// Collection of WebSites.
    /// </summary>
    SiteList sites = new SiteList();

    /// <summary>
    /// Program Entry Point.
    /// </summary>
    /// <remarks>
    /// Loads Web Sites and begins program.
    /// </remarks>
    public static void Main()
    {
        SiteManager mgr = new SiteManager();

        mgr.sites = new SiteList();
         mgr.sites[mgr.sites.NextIndex] = new WebSite(
                            "Joe",
                            "http://www.mysite.com", 
                            "Great Site.");
         mgr.sites[mgr.sites.NextIndex] = new WebSite(
                            "Don", 
                            "http://www.dondotnet.com", 
                            "Must See.");
         mgr.sites[mgr.sites.NextIndex] = new WebSite(
                            "Bob", 
                            "www.bob.com", 
                            "No http://");
 
         mgr.ShowMenu();
     }
 
     /// <summary>
     /// Shows Console Menu
     /// </summary>
     /// <remarks>
     /// Let's user make choice and performs action.
     /// </remarks>
     public void ShowMenu()
     {
         string choice;
 
         do
         {
             Console.WriteLine("Web Site Editor\n");
 
             Console.WriteLine("A - Add");
             Console.WriteLine("D - Delete");
             Console.WriteLine("M - Modify");
             Console.WriteLine("R - Report");
             Console.WriteLine("Q - Quit");
 
             Console.Write("\nPlease Choose:  ");
 
             choice = Console.ReadLine();
 
             switch (choice.ToUpper()) 
             {
                 case "A":
                     AddSite();
                     break;
                 case "D":
                     DeleteSite();
                     break;
                 case "M":
                     ModifySite();
                     break;
                 case "R":
                     ViewSites();
                     break;
                 case "Q":
                     choice = "Q";
                     break;
                 default:
                     Console.WriteLine(
     "({0})? Err...That's not what I expected.", choice);
					 break;
             }
         } while (choice != "Q");
     }
 
     /// <summary>
    /// Adds a Web Site
    /// </summary>
    /// <remarks>
    /// Prompts user for info and adds site to collection.
    /// </remarks>
    private void AddSite()
    {
        string siteName;
        string url;
        string description;

        Console.Write("Please Enter Site Name: ");
        siteName = Console.ReadLine();

        Console.Write("Please Enter URL: ");
        url = Console.ReadLine();

        Console.Write("Please Enter Description: ");
        description = Console.ReadLine();

        sites[sites.NextIndex] = new WebSite(siteName,
                                             url,
                                             description);
    }

    /// <summary>
    /// Deletes Web Site
    /// </summary>
    /// <remarks>
    /// Prints sites, get's user's choice,
    /// and deletes Web Site from collection
    /// </remarks>
    private void DeleteSite()
    {
        string choice;

        do {
            Console.WriteLine("\nDeletion Menu\n");

            DisplayShortList();

            Console.Write(
                "\nPlease select an item to delete: ");

            choice = Console.ReadLine();

            if (choice == "Q" || choice == "q")
                break;

            if (Convert.ToInt32(choice) <= sites.NextIndex)
                sites.Remove(Convert.ToInt32(choice)-1);

        } while (true);
    }

    /// <summary>
    /// Modifies a Web Site
    /// </summary>
    /// <remarks>
    /// No Implementation Yet.
    /// </remarks>
    private void ModifySite()
    {
        Console.WriteLine("Modifying Sites.");
    }

    /// <summary>
    /// View Web Sites
    /// </summary>
    /// <remarks>
    /// Prints list of Web Sites to Console.
    /// </remarks>
    private void ViewSites()
    {
        Console.WriteLine("");
        for (int i=0; i < sites.NextIndex; i++)
        {
            Console.WriteLine("Site: {0}", 
                              sites[i].ToString());
        }
        Console.WriteLine("");
    }

    /// <summary>
    /// View Numbered List
    /// </summary>
    /// <remarks>
    /// Used by the deletion operation to let user
    /// select specific Web Site to delete.
    /// </remarks>
    private void DisplayShortList()
    {
        for (int i=0; i < sites.NextIndex; i++)
        {
            Console.WriteLine("{0} - {1}", 
                              i+1, 
                              sites[i].ToString());
        }
        Console.WriteLine("Q - Quit (Back To Main Menu)");
    }
}
