    using System;
	using System.Collections;

/// <summary>
///    Describes a single web site.
/// </summary>
public class WebSite {

	const string http      = "http://";
	
	public static readonly string currentDate = new DateTime().ToString();
	
	string siteName;
	string url;
	string description;
	
	public WebSite() 
		: this("No Site", "no.url", "No Description") {}
		
	public WebSite(string newSite) 
		: this(newSite, "no.url", "No Description") {}
		
	public WebSite(string newSite, string newURL) 
		: this(newSite, newURL, "No Description") {}
	
	public WebSite(string newSite, string newURL, string newDesc)
	{
		SiteName    = newSite;
		URL         = newURL;
		Description = newDesc;
	}
	
	public override string ToString()
	{
		return siteName + ", " + url + ", " + description;
	}
	
	public override bool Equals(object evalString)
	{
		return this.ToString() == evalString.ToString();
	}
	
	public override int GetHashCode()
	{
		return this.ToString().GetHashCode();
	}
	
	protected string ValidateUrl(string url) 
	{
		if (!(url.StartsWith(http)))
		{
			return http + url;
		}
			
		return url;
	}
	
	public string SiteName
	{
		get
		{
			return siteName;
		}
		set
		{
			siteName = value;
		}
	}
	
	public string URL
	{
		get
		{
			return url;
		}
		set
		{
			url = ValidateUrl(value);
		}
	}
	
	public string Description
	{
		get
		{
			return description;
		}
		set 
		{
			description = value;
		}
	}
	
	~WebSite() {}
	   
}

/// <summary>
///    This object holds a collection of sites.
/// </summary>
public class SiteList {
	
	protected SortedList sites;
	
	public SiteList() 
	{
		sites = new SortedList();
	}
	

	public int NextIndex 
	{
		get 
		{
			return sites.Count;
		}
	}
	
	public WebSite this[int index] 
	{
		get 
		{
			if (index > sites.Count)
				return (WebSite)null;
				
			return (WebSite) sites.GetByIndex(index);
		}
		set 
		{
			if ( index < 10 )
				sites[index] = value;
		}
	}
	
	public void Remove(int element)
	{
		sites.RemoveAt(element);
	}
	
}
