using System;
using System.Collections;

public delegate void ChangeRegistrar(object sender, object evnt); 

public interface IShareTrade 
{
	decimal PricePerTrade
	{
		get ;
		set ;
	}

	event ChangeRegistrar PriceChange ;
}

//public interface IBroker 
public interface IBroker : IShareTrade
{
	string GetRating(string stock) ;
/* Part of ShareTrade
	decimal PricePerTrade
	{
		get ;
		set ;
	}
*/
	decimal this[string StockName]
	{
		get ;
		set ;
	}

//  Part of ShareTrade
//	event ChangeRegistrar PriceChange ;

}


public interface IAdvisor 
{
	string GetRating(string stock) ;

	decimal HourlyFees 
	{
		get ;
		set ;
	}

	decimal this[string StockName]
	{
		get ;
		set ;
	}
}

public class FinancialAdvisor : IBroker, IAdvisor
{
	Hashtable stocks = new Hashtable();
	decimal pricePerTrade;
	decimal fee;
	string  brokerName;

	public FinancialAdvisor() : this(13.59m, 11.73m, "Anonymous")
	{
	}

	public FinancialAdvisor(decimal tradePrice, decimal fee, string brokerName) 
	{
		pricePerTrade   = tradePrice;
		this.fee        = fee;
		this.brokerName = brokerName;
	}

	string IBroker.GetRating(string stock) 
	{
		Stock myStock = (Stock) stocks[stock];
		return Enum.GetName(typeof(StockRating), myStock.Rating);
	}

	string IAdvisor.GetRating(string stock) 
	{
		Stock myStock = (Stock) stocks[stock];
		return Enum.GetName(typeof(StockRating), (((int)++myStock.Rating)%5));
	}

	private StockRating AssignRating(Stock newStock) 
	{
		Random myRand = new Random();

		int nextRating = myRand.Next(4);
		return (StockRating) Enum.ToObject(typeof(StockRating), nextRating);
	}

	decimal IAdvisor.HourlyFees
	{
		get 
		{
			return fee;
		}
		set 
		{
			fee = value;
		}
	}

	public decimal PricePerTrade
	{
		get 
		{
			return pricePerTrade;
		}
		set 
		{
			pricePerTrade = value;
			PriceChange(brokerName, value);
		}
	}

	public decimal this[string StockName]
	{
		get 
		{
			Stock myStock = (Stock)stocks[StockName];
			return myStock.Price;
		}
		set 
		{
			Stock myStock  = new Stock();
			myStock.Name   = StockName;
			myStock.Price  = value;
			myStock.Rating = AssignRating(myStock);

			stocks.Add(StockName, myStock);
			
			//Console.WriteLine("");
			//Console.Write("Name: {0},", myStock.Name);
			//Console.WriteLine(" Rating: {1}.", GetRating(StockName));
		}
	}

	public event ChangeRegistrar PriceChange;
}

    /// <summary>
    ///    Summary description for Class1.
    /// </summary>
public class FinanceCompany : IBroker
{
	Hashtable hash = new Hashtable();
	decimal pricePerTrade;

	public FinanceCompany() : this(10.50m)
	{
	}

	public FinanceCompany(decimal price)
	{
		pricePerTrade = price;
	}

	public string GetRating(string stock) 
	{
		return "Buy";
	}

	public decimal PricePerTrade
	{
		get 
		{
			return pricePerTrade;
		}
		set 
		{
			pricePerTrade = value;
			PriceChange("FinanceBroker", value);
		}
	}

	public decimal this[string StockName]
	{
		get 
		{
			return (decimal)hash[StockName];
		}
		set 
		{
			hash.Add(StockName, value);
		}
	}

	public event ChangeRegistrar PriceChange;
}

public enum StockRating 
{
	Buy=0, Accumulate, Hold, Sell
}

public struct Stock 
{
	private string name;
	private decimal price;
	private StockRating rating;

	public string Name 
	{
		get 
		{
			return Name;
		}
		set 
		{
			name = value;
		}
	}

	public StockRating Rating 
	{
		get 
		{
			return rating;
		}
		set 
		{
			rating = value;
		}
	}
	public decimal Price 
	{
		get 
		{
			return price;
		}
		set 
		{
			price = value;
		}
	}
}

//public class StockBroker : IBroker
public class StockBroker
{
	Hashtable stocks = new Hashtable();
	decimal pricePerTrade;
	string  brokerName;

	public StockBroker() : this(13.59m, "Anonymous")
	{
	}

	public StockBroker(decimal price) : this(price, "Anonymous")
	{
	}

	public StockBroker(decimal price, string brokerName) 
	{
		pricePerTrade = price;
		this.brokerName = brokerName;
	}

	public string GetRating(string stock) 
	{
		Stock myStock = (Stock) stocks[stock];
		return Enum.GetName(typeof(StockRating), myStock.Rating);
	}

	private StockRating AssignRating(Stock newStock) 
	{
		Random myRand = new Random();

		int nextRating = myRand.Next(4);
		return (StockRating) Enum.ToObject(typeof(StockRating), nextRating);
	}

	public decimal PricePerTrade
	{
		get 
		{
			return pricePerTrade;
		}
		set 
		{
			pricePerTrade = value;
			PriceChange(brokerName, value);
		}
	}

	public decimal this[string StockName]
	{
		get 
		{
			Stock myStock = (Stock)stocks[StockName];
			return myStock.Price;
		}
		set 
		{
			Stock myStock  = new Stock();
			myStock.Name   = StockName;
			myStock.Price  = value;
			myStock.Rating = AssignRating(myStock);

			stocks.Add(StockName, myStock);
			
			//Console.WriteLine("");
			//Console.Write("Name: {0},", myStock.Name);
			//Console.WriteLine(" Rating: {1}.", GetRating(StockName));
		}
	}

	public event ChangeRegistrar PriceChange;
}


public class InterfaceTester 
{
	public static int Main(string[] args)
	{
		/* Polymorphic Interfaces
		string recommendation;
		ArrayList Brokers = new ArrayList();

		Brokers.Add(new FinanceCompany(7.32m));
		Brokers.Add(new StockBroker(11.51m, "Gofer Broke"));

		InterfaceTester iftst = new InterfaceTester();

		foreach(IBroker broker in Brokers)
		{
			broker.PriceChange += new ChangeRegistrar(iftst.PricePerTradeChange);

			broker["ABC"] = 15.39m;
			broker["DEF"] = 37.51m;
			
			Console.WriteLine("");

			Console.WriteLine("ABC Price is {0}", broker["ABC"]);
			Console.WriteLine("DEF Price is {0}", broker["DEF"]);
			
			Console.WriteLine("");

			broker.PricePerTrade = 10.55m;

			Console.WriteLine("");

			recommendation = broker.GetRating("ABC");

			Console.WriteLine("Broker's recommendation for ABC is {0}", recommendation);
		}

		Console.ReadLine();

		return 0;
		*/

		/* Single Interfaces
		FinanceCompany  finco = new FinanceCompany();
		InterfaceTester iftst = new InterfaceTester();

		finco.PriceChange += new ChangeRegistrar(iftst.PricePerTradeChange);

		finco["ABC"] = 15.39m;
		finco["DEF"] = 37.51m;

		Console.WriteLine("ABC Price is {0}", finco["ABC"]);
		Console.WriteLine("DEF Price is {0}", finco["DEF"]);

		Console.WriteLine("");

		finco.PricePerTrade = 10.55m;

		Console.WriteLine("");

		string recommendation = finco.GetRating("ABC");

		Console.WriteLine("finco's recommendation for ABC is {0}", recommendation);

		Console.ReadLine();

		return 0;
		*/
		string recommendation;

		FinancialAdvisor  finad = new FinancialAdvisor();
		InterfaceTester iftst = new InterfaceTester();

		finad.PriceChange += new ChangeRegistrar(iftst.PricePerTradeChange);

		finad["ABC"] = 15.39m;
		finad["DEF"] = 37.51m;

		Console.WriteLine("ABC Price is {0}", finad["ABC"]);
		Console.WriteLine("DEF Price is {0}", finad["DEF"]);

		Console.WriteLine("");

		finad.PricePerTrade = 10.55m;

		// HourlyFees property is hidden and won't compile
		//finad.HourlyFees    = 9.00m;

		Console.WriteLine("");

		recommendation = ((IBroker) finad).GetRating("ABC");
		
		Console.WriteLine("(IBroker)finad's recommendation for ABC is {0}", recommendation);
		
		recommendation = ((IAdvisor)finad).GetRating("ABC");

		Console.WriteLine("(IAdvisor)finad's recommendation for ABC is {0}", recommendation);

		Console.ReadLine();

		return 0;

	}

	public void PricePerTradeChange(object sender, object evnt) 
	{
		Console.WriteLine("Trading price for {0} changed to {1}.",
			(string) sender, (decimal) evnt);
	}
}

public class Accountant : StockBroker, IBroker 
{
	
}