using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

public class CiteManagerForm : Form
{
    private SiteManager citeMgr;

    private Container components;
    private Button    addButton;
    private Button    deleteButton;
    private Button    modifyButton;
    private Button    viewButton;
    private Label     selectLabel;

    private MainMenu  citeMgrMenu;
    private MenuItem  fileMenu;
    private MenuItem  fileOpenMenu;
    private MenuItem  fileSaveMenu;
    private MenuItem  fileSeparatorMenu;
    private MenuItem  fileExitMenu;

    public CiteManagerForm()
    {
        InitializeComponent();
        citeMgr = new SiteManager();
    }

    public override void Dispose()
    {
        base.Dispose();
        components.Dispose();
    }

    private void InitializeComponent()
    {
        components   = new Container();
        addButton    = new Button();
        deleteButton = new Button();
        modifyButton = new Button();
        viewButton   = new Button();
        selectLabel  = new Label();

        citeMgrMenu       = new MainMenu();
        fileMenu          = new MenuItem();
        fileOpenMenu      = new MenuItem();
        fileSaveMenu      = new MenuItem();
        fileSeparatorMenu = new MenuItem();
        fileExitMenu      = new MenuItem();

        addButton.Text     = "Add";
        addButton.Click   +=
            new EventHandler (addButton_Click);
        addButton.Location = new Point(94, 80);
        addButton.Size     = new Size(75, 23);
        addButton.TabIndex = 0;

        deleteButton.Text     = "Delete";
        deleteButton.Click   +=
            new EventHandler(deleteButton_Click);
        deleteButton.Location = new Point(94, 128);
        deleteButton.Size     = new Size(75, 23);
        deleteButton.TabIndex = 1;

        modifyButton.Text     = "Modify";
        modifyButton.Click   +=
            new EventHandler(modifyButton_Click);
        modifyButton.Location = new Point(94, 176);
        modifyButton.Size     = new Size(75, 23);
        modifyButton.TabIndex = 2;

        viewButton.Text     = "View";
        viewButton.Click   +=
            new EventHandler(viewButton_Click);
        viewButton.Location = new Point(94, 224);
        viewButton.Size     = new Size(75, 23);
        viewButton.TabIndex = 3;

        selectLabel.Text        = "Please Make a Selection:";
        selectLabel.Font        = new Font("Lucida Console",
                                            12,
                                            FontStyle.Italic);
        selectLabel.Location    = new Point(10, 32);
        selectLabel.Size        = new Size(246, 18);
        selectLabel.BorderStyle =
            System.Windows.Forms.BorderStyle.Fixed3D;
        selectLabel.AutoSize    = true;
        selectLabel.TabIndex    = 4;

        fileMenu.Text           = "&File";
        fileMenu.Index          = 0;

        fileOpenMenu.Text       = "&Open";
        fileOpenMenu.Click     +=
            new EventHandler (fileOpenMenu_Click);
        fileOpenMenu.Index      = 0;

        fileSaveMenu.Text       = "&Save";
        fileSaveMenu.Click     +=
            new EventHandler (fileSaveMenu_Click);
        fileSaveMenu.Index      = 1;

        fileSeparatorMenu.Text  = "-";
        fileSeparatorMenu.Index = 2;

        fileExitMenu.Text       = "E&xit";
        fileExitMenu.Click     +=
            new EventHandler (fileExitMenu_Click);
        fileExitMenu.Index      = 3;

        fileMenu.MenuItems.Add(fileOpenMenu);
        fileMenu.MenuItems.Add(fileSaveMenu);
        fileMenu.MenuItems.Add(fileSeparatorMenu);
        fileMenu.MenuItems.Add(fileExitMenu);

        citeMgrMenu.MenuItems.Add(fileMenu);

        Menu       = citeMgrMenu;
        Text       = "Cite Manager";
        ClientSize = new Size (264, 273);

        Controls.Add(selectLabel);
        Controls.Add(modifyButton);
        Controls.Add(viewButton);
        Controls.Add(deleteButton);
        Controls.Add(addButton);
    }

    protected void deleteButton_Click (object sender,
                                       System.EventArgs e)
    {
        DeleteForm df = new DeleteForm(citeMgr);

        df.ShowDialog(this);
    }

    protected void modifyButton_Click (object sender,
                                       System.EventArgs e)
    {
        ModifyForm mf = new ModifyForm(citeMgr);

        mf.ShowDialog(this);
    }

    protected void viewButton_Click (object sender,
                                     System.EventArgs e)
    {
        ViewForm vf = new ViewForm(citeMgr);
        vf.ShowDialog(this);
    }

    protected void addButton_Click (object sender,
                                    System.EventArgs e)
    {
        AddForm af = new AddForm();

        DialogResult dlgRes = af.ShowDialog(this);

        switch (dlgRes)
        {
            case DialogResult.OK:
                citeMgr.AddSite(
                    af.sitenameTextbox.Text,
                    af.addressTextbox.Text,
                    af.descriptionTextbox.Text);
                break;
            case DialogResult.Cancel:
                // do nothing
                break;
            default:
                break;
        }
    }

    protected void fileOpenMenu_Click (object sender,
                                       System.EventArgs e)
    {
        MessageBox.Show("File/Open Clicked", 
                        "Menu Selection",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information);
    }

    protected void fileSaveMenu_Click (object sender,
                                       System.EventArgs e)
    {
        MessageBox.Show("File/Save Clicked",
                        "Menu Selection",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Information);
    }

    protected void fileExitMenu_Click (object sender,
                                       System.EventArgs e)
    {
        DialogResult dlgRes = MessageBox.Show(
            "Are You Sure?",
            "Program Exiting",
            MessageBoxButtons.OKCancel,
            MessageBoxIcon.Warning);

        if (dlgRes == DialogResult.OK)
            Close();
    }

    public static void Main(string[] args)
    {
        Application.Run(new CiteManagerForm());
    }
}
