using System;
using System.Data;
using System.Data.SqlClient;

	/// <summary>
	///		Program demonstrating creating and
	///		execution of stored procedures.
	/// </summary>
class StoredProcedures
{
	// print shippers table to console
	void PrintReport() 
	{
		SqlDataReader dbReader  = null;
		SqlConnection conn = null;

		try 
		{
			conn = new SqlConnection("Data Source=localhost;User Id=sa;Password=pwd;Initial Catalog=northwind");
			//conn = new SqlConnection("Data Source=localhost;trusted_connection=yes;Initial Catalog=Northwind");

			SqlCommand cmd = conn.CreateCommand();
			cmd.CommandText = "SELECT * FROM Shippers";

			conn.Open();

			dbReader = cmd.ExecuteReader();

			Console.WriteLine();
			
			for (int i=0; i < dbReader.FieldCount; i++) 
			{
				Console.Write("{0}", dbReader.GetName(i).PadLeft(20, ' '));
			}
			Console.WriteLine("\n{0}", "".PadLeft(60, '-'));

			int    index;
			string companyName;
			string phone;

			while (dbReader.Read())
			{
				index       = dbReader.GetInt32(0);
				companyName = (string)dbReader.GetValue(1);
				phone       = (string)dbReader["Phone"];

				Console.WriteLine("{0}{1}{2}", 
					index.ToString().PadLeft(20, ' '),
					companyName.PadLeft(20, ' '),
					phone.PadLeft(20, ' '));
			}
		}
		catch (SqlException sqle) 
		{
			Console.WriteLine("SqlException: {0}", sqle.Message);
		}
		catch (Exception e) 
		{
			Console.WriteLine("Generic Exception: {0}", e.Message);
		}
		finally 
		{
			if (dbReader != null) 
			{
				dbReader.Close();
				conn.Close();
			}
		}
	}

	// create a stored procedure in the
	// Northwind Traders database
	void CreateStoredProcedure() 
	{
		SqlConnection conn = null;

		try 
		{
			conn = new SqlConnection("Data Source=localhost;User Id=sa;Password=pwd;Initial Catalog=northwind");

			conn.Open();

			SqlCommand nqcmd = conn.CreateCommand();
			nqcmd.CommandText  = 
				@"
					CREATE PROCEDURE GetPhoneNumber
						@companyName nvarchar(20),
						@phone       nvarchar(20) output
					AS
					SELECT @phone = phone FROM Shippers
					WHERE companyName = @companyName
				";
				
			nqcmd.ExecuteNonQuery();
		}
		catch (SqlException sqle) 
		{
			Console.WriteLine("CreateStoredProcedure SqlException: {0}", sqle.Message);
		}
		catch (Exception e) 
		{
			Console.WriteLine("Generic Exception: {0}", e.Message);
		}
		finally 
		{
			if (conn != null) 
			{
				conn.Close();
			}
		}
	}

	// run a stored procedure
	void ExecuteStoredProcedure() 
	{
		SqlConnection conn      = null;
		SqlDataReader dbReader  = null;

		try 
		{
			conn = new SqlConnection("Data Source=localhost;User Id=sa;Password=pwd;Initial Catalog=northwind");

			conn.Open();

			SqlCommand cmd = new SqlCommand("GetPhoneNumber", conn);
			cmd.CommandType = CommandType.StoredProcedure;

			SqlParameter inParam  = new SqlParameter("@companyName", SqlDbType.NChar, 20);
			inParam.Direction     = ParameterDirection.Input;
			inParam.Value         = "United Package";

			SqlParameter outParam = new SqlParameter("@phone", SqlDbType.NChar, 20);
			outParam.Direction    = ParameterDirection.Output;

			cmd.Parameters.Add(inParam);
			cmd.Parameters.Add(outParam);

			dbReader = cmd.ExecuteReader();

			Console.WriteLine("United Package Phone #: {0}", cmd.Parameters["@phone"].Value);

			SqlCommand nqcmd = conn.CreateCommand();
			nqcmd.CommandText  = "DROP PROCEDURE GetPhoneNumber";
				
			nqcmd.ExecuteNonQuery();

		}
		catch (SqlException sqle) 
		{
			Console.WriteLine("ExecuteStoredProcedure SqlException: {0}", sqle.Message);
		}
		catch (Exception e) 
		{
			Console.WriteLine("Generic Exception: {0}", e.Message);
		}
		finally 
		{
			if (dbReader != null) 
			{
				dbReader.Close();
				conn.Close();
			}
		}
	}

	// create and execute a stored procedure
	static void Main(string[] args)
	{
		StoredProcedures stProc = new StoredProcedures();

		stProc.CreateStoredProcedure();

		stProc.ExecuteStoredProcedure();

		stProc.PrintReport();

		Console.ReadLine();
	}
}
