using System;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Http;
using System.Runtime.Remoting.Lifetime;
using BasicServer;

	/// <summary>
	/// Remoting Object Lifetime Demo.
	/// </summary>
class LeasingDemo : ISponsor
{
	ILease lease;

	// implement lease logic
	public void ImplementLease() 
	{
		BasicRemotingServer brs = new BasicRemotingServer();

		lease = (ILease)brs.InitializeLifetimeService();
		lease.Register(this, new TimeSpan(0, 0, 3));

		PrintLeaseInfo();

		Console.WriteLine("\nServer Response: {0}", 
			brs.getServerResponse());

		Console.ReadLine();
	}

	// ISponsor.Renewal - called to renew lease
	public TimeSpan Renewal(ILease myLease)
	{
		TimeSpan timeSpan = new TimeSpan(0, 0, 3);

		Console.WriteLine("\nLease Renewed.\n");
		PrintLeaseInfo();

		return timeSpan;
	}

	// print lease info
	void PrintLeaseInfo() 
	{
		if (lease != null) 
		{
			Console.WriteLine("Lease Info\n");

			Console.WriteLine("  CurrentLeaseTime: {0}", lease.CurrentLeaseTime );
			Console.WriteLine("  InitialLeaseTime: {0}", lease.InitialLeaseTime);
			Console.WriteLine("   RenewOnCallTime: {0}", lease.RenewOnCallTime);
			Console.WriteLine("SponsorshipTimeout: {0}", lease.SponsorshipTimeout);
		}
	}

	// entry point
	static void Main(string[] args)
	{
		ChannelServices.RegisterChannel(new HttpChannel());

		LeasingDemo leaseDemo = new LeasingDemo();
		leaseDemo.ImplementLease();
	}

}
