using System;
using System.Threading;

/// <summary>
///		Synchronized data.
/// </summary>
class SyncData
{
	int index = 0;

	string[] comment = new string[]
		{ "one", "two",   "three", "four", "five", 
		  "six", "seven", "eight", "nine", "ten" };

	public string GetNextComment() 
	{
		// allow only a single thread at a time
		lock (this) 
		{
			Thread.Sleep(1);
			if (index < comment.Length) 
			{
				return comment[index++];
			}
			else 
			{
				return "empty";
			}
		}
	}
}

/// <summary>
///		Demonstrates synchronized data access.
/// </summary>
class Synchronization
{
	SyncData sdat = new SyncData();

	static void Main(string[] args)
	{
		Synchronization sync = new Synchronization();

		Thread t1 = new Thread(new ThreadStart(sync.GetComments));
		Thread t2 = new Thread(new ThreadStart(sync.GetComments));
		Thread t3 = new Thread(new ThreadStart(sync.GetComments));

		t1.Name = "Thread 1";
		t2.Name = "Thread 2";
		t3.Name = "Thread 3";

		t1.Start();
		t2.Start();
		t3.Start();

		Console.ReadLine();
	}

	public void GetComments() 
	{
		string comment;

		do 
		{
			comment = sdat.GetNextComment();

			Console.WriteLine("Current Thread: {0}, comment: {1}", 
				Thread.CurrentThread.Name, comment);

		} while (comment != "empty");
	}
}
