using System;
using System.Collections;
using System.Text;

/// <summary>
///    Shows how to use the String class.
/// </summary>
class TheStringShow
{
	static void Main(string[] args)
	{
		int    intResult;
		char   charResult;
		bool   boolResult;
		string stringResult;

		String str1 = "string1";
		String str2 = "string2";

		// string comparison
		intResult = String.Compare(str1, str2);

		Console.WriteLine("String.Compare({0}, {1}) = {2}\n",
			str1, str2, intResult);

		// locale independent comparison
		intResult = String.CompareOrdinal(str2, str1);

		Console.WriteLine("String.CompareOrdinal({0}, {1}) = {2}\n",
			str2, str1, intResult);

		// append one string to another
		stringResult = String.Concat(str1, str2);

		Console.WriteLine("String.Concat({0}, {1}) = {2}\n",
			str1, str2, stringResult);

		// make a copy of a string
		stringResult = String.Copy(str1);

		Console.WriteLine("String.Copy({0}) = {1}\n",
			str1, stringResult);

		// check equality of two strings
		boolResult = String.Equals(str1, str2);

		Console.WriteLine("String.Equals({0}, {1}) = {2}\n",
			str1, str2, boolResult);

		// apply formatting to a string
		String formatString = "{0,15}";
		stringResult = String.Format(formatString, str2);

		Console.WriteLine("String.Format({0}, {1}) = [{2}]\n",
			formatString, str2, stringResult);

		// show how to intern a string
		String objStr1      = String.Concat("string", "1");
		String internedStr1 = String.Intern(objStr1);

		Console.WriteLine("(object)objStr1 == (object)str1 is {0}\n",
			((object)objStr1 == (object)str1));

		Console.WriteLine("(object)internedStr1 == (object)str1 is {0}\n",
			((object)internedStr1 == (object)str1));

		// determine if a string is interned
		stringResult = String.IsInterned(internedStr1);

		Console.WriteLine("String.IsInterned({0}) = {1}\n",
			internedStr1, stringResult);

		// join a couple strings with a comma separator
		String[] strArr = new String[] { str1, str2 };
		stringResult = String.Join(",", strArr);

		Console.WriteLine("String.Join(\",\", [str1 and str2]) = {0}\n",
			stringResult);

		// return this string
		stringResult = (String)str1.Clone();

		Console.WriteLine("(String){0}.Clone() = {1}\n",
			str1, stringResult);

		// compare two strings thru instance
		intResult = str1.CompareTo(str2);

		Console.WriteLine("{0}.CompareTo({1}) = {2}\n",
			str1, str2, intResult);

		// copy string to char array
		char[] charArr = new char[str1.Length];
		str1.CopyTo(0, charArr, 0, str1.Length);

		Console.WriteLine("{0}.CopyTo(0, charArr, 0, str1.Length) = ",
			str1);

		foreach(char character in charArr) 
		{
			Console.Write("{0}  ", character);
		}
		Console.WriteLine("\n");

		// check the ending of a string
		boolResult = str1.EndsWith("2");

		Console.WriteLine("{0}.EndsWith(\"2\") = {1}\n",
			str1, boolResult);

		// check strings for equality
		boolResult = str1.Equals(str2);

		Console.WriteLine("{0}.Equals({1}) = {2}\n",
			str1, str2, boolResult);

		// get an enumerator
		CharEnumerator charEnum = str1.GetEnumerator();

		Console.WriteLine("charEnum is IEnumerator: {0}",
			charEnum is IEnumerator);

		// get index of a character in a string
		intResult = str1.IndexOf('1');

		Console.WriteLine("str1.IndexOf('1'): {0}",
			intResult);

		// insert a character into a string
		stringResult = str2.Insert(6, "1");

		Console.WriteLine("str2.Insert(6, \"1\"): {0}",
			stringResult);

		// get index of a character in a string
		string stateString = "Mississippi";
		intResult = stateString.LastIndexOf('s');

		Console.WriteLine("stateString.LastIndexOf('s'): {0}",
			intResult);

		// pad characters to the left of a string
		stringResult = str1.PadLeft(15);

		Console.WriteLine("str1.PadLeft(15): [{0}]",
			stringResult);

		// pad characters to the right of a string
		stringResult = str1.PadRight(15, '*');

		Console.WriteLine("str1.PadRight(15, '*'): [{0}]",
			stringResult);

		// remove a portion of a string
		stringResult = str2.Remove(3, 3);

		Console.WriteLine("str2.Remove(3, 3): {0}",
			stringResult);

		// replace a portion of a string
		stringResult = str2.Replace('2', '5');

		Console.WriteLine("str2.Replace('2', '5'): {0}",
			stringResult);

		// splitting a string
		String csvString = "one, two, three";

		string[] stringArray = csvString.Split(new char[] {','});

		foreach( string strItem in stringArray ) 
		{
			Console.WriteLine("Item: {0}", strItem);
		}

		// see if a string starts with a string
		boolResult = str1.StartsWith("Str");

		Console.WriteLine("str1.StartsWith(\"Str\"): {0}",
			boolResult);

		// retrieve a substring
		stringResult = str1.Substring(3);

		Console.WriteLine("str1.Substring(3): {0}",
			stringResult);

		// move string to a character array
		char[] characterArray = str1.ToCharArray();

		foreach( char character in charArr ) 
		{
			Console.WriteLine("Char: {0}", character);
		}
		Console.WriteLine();

		// convert a string to lower case
		string ucString = "UpperCaseString";

		stringResult = ucString.ToLower();

		Console.WriteLine("ucString.ToLower(): {0}",
			stringResult);

		// convert a string to upper case
		stringResult = str1.ToUpper();

		Console.WriteLine("str1.ToUpper(): {0}",
			stringResult);

		// trim a string on both sides
		string trimString = "   nonwhitespace   ";

		stringResult = trimString.Trim();

		Console.WriteLine("trimString.Trim(): [{0}]",
			stringResult);

		// trim a string on the right
		stringResult = trimString.TrimEnd(new char[] {' '});

		Console.WriteLine("trimString.TrimEnd(): [{0}]",
			stringResult);

		// trim a string on the left
		stringResult = trimString.TrimStart(new char[] {' '});

		Console.WriteLine("trimString.TrimStart(): [{0}]",
			stringResult);

		// get the length of a string
		intResult = str1.Length;

		Console.WriteLine("str1.Length: {0}",
			intResult);

		// get a specific character
		charResult = str1[3];

		Console.WriteLine("str1[3]: {0}",
			charResult);

		//
		// StringBuilder Methods
		//

		// Append a string
		StringBuilder myStringBuilder;

		myStringBuilder = new StringBuilder("Original");

		myStringBuilder.Append("Appended");

		Console.WriteLine(
            "myStringBuilder.Append(\"Appended\"): {0}",
			myStringBuilder);

		// append a format to a string
		myStringBuilder = new StringBuilder("Original");

		myStringBuilder.AppendFormat("{0,10}", "Appended");

		Console.WriteLine(
"myStringBuilder.AppendFormat(\"{0,10}\",\"Appended\"): {0}",
			myStringBuilder);

		// guarantee a minimum capacity
		int capacity;

		myStringBuilder = new StringBuilder();

		capacity = myStringBuilder.EnsureCapacity(129);

		Console.WriteLine(
			"myStringBuilder.EnsureCapacity(129): {0}",
			capacity);

		// compare two StringBuilders
		StringBuilder anotherStringBuilder;

		myStringBuilder = new StringBuilder("my string builder");
		anotherStringBuilder = new StringBuilder("another string builder");

		boolResult = myStringBuilder.Equals(anotherStringBuilder);

		Console.WriteLine(
"myStringBuilder.Equals(anotherStringBuilder): {0}",
boolResult);

		// insert a string
		myStringBuilder = new StringBuilder("one, three");

		myStringBuilder.Insert(3, ", two");

		Console.WriteLine(
			"myStringBuilder.Insert(3, \", two\"): {0}",
			myStringBuilder);

		// remove a string
		myStringBuilder = new StringBuilder("Jane X. Doe");

		myStringBuilder.Remove(4, 3);

		Console.WriteLine(
			"myStringBuilder.Remove(4, 3): {0}",
			myStringBuilder);

		// replace characters
		myStringBuilder = new StringBuilder("Jane X. Doe");

		myStringBuilder.Replace('X', 'B');

		Console.WriteLine(
			"myStringBuilder.Replace('X', 'B'): {0}",
			myStringBuilder);

		// create a string
		myStringBuilder = new StringBuilder("my string");

		stringResult = myStringBuilder.ToString();

		Console.WriteLine(
			"myStringBuilder.ToString(): {0}",
			stringResult);

		// get the capacity of a string
		myStringBuilder = new StringBuilder("my string");

		intResult = myStringBuilder.Capacity;

		Console.WriteLine(
			"myStringBuilder.Capacity: {0}",
			intResult);

		// get the length of a string
		myStringBuilder = new StringBuilder("my string");

		intResult = myStringBuilder.Length;

		Console.WriteLine(
			"myStringBuilder.Length: {0}",
			intResult);

		// get the max capacity of a string
		myStringBuilder = new StringBuilder("my string");

		intResult = myStringBuilder.MaxCapacity;

		Console.WriteLine(
			"myStringBuilder.MaxCapacity: {0}",
			intResult);

		// use the indexer
		myStringBuilder = new StringBuilder("my string");

		charResult = myStringBuilder[1];

		Console.WriteLine(
			"myStringBuilder[1]: {0}",
			charResult);

		//
		// Formatting
		//
		Console.WriteLine("Hex: {0:x}", 255);

		Console.WriteLine("Million: {0:$#,#.00}", 1000000);

		Console.ReadLine();

	}
}
