namespace WebSites
{
	using System;
	using System.Collections;

	/// <summary>
	///    Describes a single web site.
	/// </summary>
	public class WebSite 
	{
		public string siteName;
		public string url;
		public string description;
	
		public WebSite(string newSite, string newURL, string newDesc) 
		{
			siteName    = newSite;
			url         = newURL;
			description = newDesc;
		}
	
		public override string ToString() 
		{
			return siteName + ", " + url + ", " + description;
		}
	
		public override bool Equals(object evalString) 
		{
			return this.ToString() == evalString.ToString();
		}
	
		public override int GetHashCode() 
		{
			return this.ToString().GetHashCode();
		}	   
	}

	/// <summary>
	///    This object holds a collection of WebSites.
	/// </summary>
	public class SiteList : ICollection, IComparer, IEnumerable, IList
	{
		protected ArrayList sites;
	
		public SiteList() 
		{
			sites = new ArrayList();
		}
	
		// ICollection.Count
		public int Count
		{
			get 
			{
				return sites.Count;
			}
		}
	
		// ICollection.IsSynchronized
		public bool IsSynchronized
		{
			get 
			{
				return sites.IsSynchronized;
			}
		}

		// ICollection.SyncRoot
		public object SyncRoot
		{
			get 
			{
				return sites.SyncRoot;
			}
		}
	
		// ICollection.CopyTo
		public void CopyTo(Array dest, int index) 
		{
			sites.CopyTo(dest, index);
		}

		// IComparer.Compare
		public int Compare(object site1, object site2) 
		{
			return String.Compare(site1.ToString(), site2.ToString());
		}

		// IEnumerable.GetEnumerator
		public IEnumerator GetEnumerator() 
		{
			Console.WriteLine("SiteList GetEnumerator called.");
			return new SiteEnumerator(sites);
		}

		// IEnumerator class
		class SiteEnumerator : IEnumerator 
		{
			ArrayList   sites;
			IEnumerator myEnumerator;

			public SiteEnumerator(ArrayList sites) 
			{
				Console.WriteLine("SiteEnumerator Constructor called.");
				this.sites   = sites;
				myEnumerator = sites.GetEnumerator();
			}

			// IEnumerator.Current
			public object Current 
			{
				get 
				{
					Console.WriteLine("SiteEnumerator Current called.");
					return myEnumerator.Current;
				}
			}

			// IEnumerator.MoveNext
			public bool MoveNext() 
			{
				Console.WriteLine("SiteEnumerator MoveNext called.");
				return myEnumerator.MoveNext();
			}

			// IEnumerator.Reset
			public void Reset() 
			{
				Console.WriteLine("SiteEnumerator Reset called.");
			}
		}

		// IList.IsFixedSize
		public bool IsFixedSize 
		{
			get 
			{
				return true;
			}
		}

		// IList.IsReadOnly
		public bool IsReadOnly 
		{
			get 
			{
				return false;
			}
		}

		// IList Indexer
		public object this[int index]
		{
			get
			{
				if (index > sites.Count)
					return (WebSite)null;

				return (WebSite) sites[index];
			}
			set
			{
				if ( index < 10 )
					sites.Add(value);
			}
		}

		// IList.Add
		public int Add(object webSite) 
		{
			if ( sites.Count < 10 )
				return sites.Add(webSite);

			return -1;
		}

		// IList.Clear
		public void Clear() 
		{
			sites.Clear();
		}

		// IList.Contains
		public bool Contains(object webSite) 
		{
			return sites.Contains(webSite);
		}

		// IList.IndexOf
		public int IndexOf(object webSite) 
		{
			return sites.IndexOf(webSite);
		}

		// IList.Insert
		public void Insert(int index, object webSite) 
		{
			sites.Insert(index, webSite);
		}

		// IList.Remove
		public void Remove(object webSite) 
		{
			sites.Remove(webSite);
		}
	
		// IList.RemoveAt
		public void RemoveAt(int element)
		{
			if ( element < Count ) 
			{
				sites.RemoveAt(element);
			}
		}
	}
}
